@@ -7,7 +7,7 @@ use anyhow::Result;
 use mlua::Lua;
 use mlua::LuaSerdeExt;
 use mlua::SerializeOptions;
-use serde::Deserialize;
+use serde::de::DeserializeOwned;
 use serde::Serialize;
 use std::fs;
 
@@ -81,7 +81,7 @@ pub fn init(lua: &Lua) -> Result<(Config, Option<Hooks>)> {
 
     let hooks: Option<Hooks> = lua
         .load(DEFAULT_LUA_SCRIPT)
-        .set_name("xplr init")?
+        .set_name("xplr init")
         .call(())
         .and_then(|v| lua.from_value(v))?;
 
@@ -98,7 +98,7 @@ pub fn extend(lua: &Lua, path: &str) -> Result<(Config, Option<Hooks>)> {
 
     let hooks: Option<Hooks> = lua
         .load(&script)
-        .set_name(path)?
+        .set_name(path)
         .call(())
         .and_then(|v| lua.from_value(v))?;
 
@@ -138,7 +138,7 @@ pub fn resolve_fn<'lua>(
     resolve_fn_recursive(globals, path.split('.'))
 }
 
-pub fn call<'lua, R: Deserialize<'lua>>(
+pub fn call<'lua, R: DeserializeOwned>(
     lua: &'lua Lua,
     func: &str,
     arg: mlua::Value<'lua>,
@@ -160,24 +160,24 @@ mod tests {
         assert!(check_version(VERSION, "foo path").is_ok());
 
         // Current release if OK
-        assert!(check_version("0.21.2", "foo path").is_ok());
+        assert!(check_version("0.21.5", "foo path").is_ok());
 
         // Prev major release is ERR
         // - Not yet
 
         // Prev minor release is ERR (Change when we get to v1)
-        assert!(check_version("0.20.2", "foo path").is_err());
+        assert!(check_version("0.20.5", "foo path").is_err());
 
         // Prev bugfix release is OK
-        assert!(check_version("0.21.1", "foo path").is_ok());
+        assert!(check_version("0.21.4", "foo path").is_ok());
 
         // Next major release is ERR
-        assert!(check_version("1.20.2", "foo path").is_err());
+        assert!(check_version("1.20.5", "foo path").is_err());
 
         // Next minor release is ERR
-        assert!(check_version("0.22.2", "foo path").is_err());
+        assert!(check_version("0.22.5", "foo path").is_err());
 
         // Next bugfix release is ERR (Change when we get to v1)
-        assert!(check_version("0.21.3", "foo path").is_err());
+        assert!(check_version("0.21.6", "foo path").is_err());
     }
 }
