@@ -40,15 +40,14 @@ pub fn get_tty() -> Result<fs::File> {
 // returns physical path. As a workaround, this function tries to use `PWD`
 // environment variable that is configured by shell.
 fn get_current_dir() -> Result<PathBuf, std::io::Error> {
-    let cur = std::env::current_dir();
     if let Ok(pwd) = std::env::var("PWD") {
         if pwd.is_empty() {
-            cur
+            std::env::current_dir()
         } else {
             Ok(PathBuf::from(pwd))
         }
     } else {
-        cur
+        std::env::current_dir()
     }
 }
 
@@ -349,9 +348,7 @@ impl Runner {
                     while let Some(msg) = app.msg_out.pop_front() {
                         use app::MsgOut::*;
                         match msg {
-                            // NOTE: Do not schedule critical tasks via tx_msg_in in this loop.
-                            // Try handling them immediately.
-                            Enque(task) => {
+                            Enqueue(task) => {
                                 tx_msg_in.send(task)?;
                             }
 
