@@ -16,11 +16,10 @@ use std::collections::HashMap;
 use std::env;
 use std::ops::BitXor;
 use time::macros::format_description;
-use tui::backend::Backend;
 use tui::layout::Rect as TuiRect;
 use tui::layout::{Constraint as TuiConstraint, Direction, Layout as TuiLayout};
 use tui::style::{Color, Modifier as TuiModifier, Style as TuiStyle};
-use tui::text::{Span, Spans, Text};
+use tui::text::{Line, Span, Text};
 use tui::widgets::{
     Block, BorderType as TuiBorderType, Borders as TuiBorders, Cell, List, ListItem,
     Paragraph, Row, Table,
@@ -40,6 +39,15 @@ fn read_only_indicator(app: &app::App) -> &str {
     }
 }
 
+fn selection_indicator(app: &app::App) -> String {
+    let count = app.selection.len();
+    if count == 0 {
+        String::new()
+    } else {
+        format!(" {{{count} sel}}")
+    }
+}
+
 pub fn string_to_text<'a>(string: String) -> Text<'a> {
     if *NO_COLOR {
         Text::raw(string)
@@ -329,6 +337,7 @@ impl Into<TuiStyle> for Style {
             TuiStyle {
                 fg: self.fg,
                 bg: self.bg,
+                underline_color: None,
                 add_modifier: TuiModifier::from_bits_truncate(xor(self.add_modifiers)),
                 sub_modifier: TuiModifier::from_bits_truncate(xor(self.sub_modifiers)),
             }
@@ -401,22 +410,22 @@ impl Into<nu_ansi_term::Style> for Style {
             style.add_modifiers.as_ref().map_or(false, f)
         }
 
-        nu_ansi_term::Style {
-            foreground: self.fg.and_then(convert_color),
-            background: self.bg.and_then(convert_color),
-            is_bold: match_modifiers(&self, |m| m.contains(&Modifier::Bold)),
-            is_dimmed: match_modifiers(&self, |m| m.contains(&Modifier::Dim)),
-            is_italic: match_modifiers(&self, |m| m.contains(&Modifier::Italic)),
-            is_underline: match_modifiers(&self, |m| m.contains(&Modifier::Underlined)),
-            is_blink: match_modifiers(&self, |m| {
-                m.contains(&Modifier::SlowBlink) || m.contains(&Modifier::RapidBlink)
-            }),
-            is_reverse: match_modifiers(&self, |m| m.contains(&Modifier::Reversed)),
-            is_hidden: match_modifiers(&self, |m| m.contains(&Modifier::Hidden)),
-            is_strikethrough: match_modifiers(&self, |m| {
-                m.contains(&Modifier::CrossedOut)
-            }),
-        }
+        let mut style = nu_ansi_term::Style::new();
+        style.foreground = self.fg.and_then(convert_color);
+        style.background = self.bg.and_then(convert_color);
+        style.is_bold = match_modifiers(&self, |m| m.contains(&Modifier::Bold));
+        style.is_dimmed = match_modifiers(&self, |m| m.contains(&Modifier::Dim));
+        style.is_italic = match_modifiers(&self, |m| m.contains(&Modifier::Italic));
+        style.is_underline =
+            match_modifiers(&self, |m| m.contains(&Modifier::Underlined));
+        style.is_blink = match_modifiers(&self, |m| {
+            m.contains(&Modifier::SlowBlink) || m.contains(&Modifier::RapidBlink)
+        });
+        style.is_reverse = match_modifiers(&self, |m| m.contains(&Modifier::Reversed));
+        style.is_hidden = match_modifiers(&self, |m| m.contains(&Modifier::Hidden));
+        style.is_strikethrough =
+            match_modifiers(&self, |m| m.contains(&Modifier::CrossedOut));
+        style
     }
 }
 
@@ -656,8 +665,8 @@ pub fn block<'a>(config: PanelUiConfig, default_title: String) -> Block<'a> {
         .border_style(config.border_style.into())
 }
 
-fn draw_table<B: Backend>(
-    f: &mut Frame<B>,
+fn draw_table(
+    f: &mut Frame,
     screen_size: TuiRect,
     layout_size: TuiRect,
     app: &app::App,
@@ -669,6 +678,7 @@ fn draw_table<B: Backend>(
     let header_height = app_config.general.table.header.height.unwrap_or(1);
     let height: usize =
         (layout_size.height.max(header_height + 2) - (header_height + 2)).into();
+    let row_style = app_config.general.table.row.style.to_owned();
 
     let rows = app
         .directory_buffer
@@ -767,17 +777,17 @@ fn draw_table<B: Backend>(
                                     c.format.as_ref().map(|f| {
                                         let out = lua::call(lua, f, v.clone())
                                             .unwrap_or_else(|e| format!("{e:?}"));
-                                        string_to_text(out)
+                                        (string_to_text(out), c.style.to_owned())
                                     })
                                 })
-                                .collect::<Vec<Text>>()
+                                .collect::<Vec<(Text, Style)>>()
                         })
                         .unwrap_or_default()
-                        .iter()
-                        .map(|x| Cell::from(x.to_owned()))
+                        .into_iter()
+                        .map(|(text, style)| Cell::from(text).style(style.into()))
                         .collect::<Vec<Cell>>();
 
-                    Row::new(cols)
+                    Row::new(cols).style(row_style.to_owned().into())
                 })
                 .collect::<Vec<Row>>()
         })
@@ -811,8 +821,7 @@ fn draw_table<B: Backend>(
         format!("({node_count}) ")
     };
 
-    let table = Table::new(rows)
-        .widths(&table_constraints)
+    let table = Table::new(rows, table_constraints)
         .style(app_config.general.table.style.to_owned().into())
         .highlight_style(app_config.general.focus_ui.style.to_owned().into())
         .column_spacing(app_config.general.table.col_spacing.unwrap_or_default())
@@ -831,7 +840,10 @@ fn draw_table<B: Backend>(
                 .to_owned()
                 .unwrap_or_default()
                 .iter()
-                .map(|c| Cell::from(c.format.to_owned().unwrap_or_default()))
+                .map(|c| {
+                    Cell::from(c.format.to_owned().unwrap_or_default())
+                        .style(c.style.to_owned().into())
+                })
                 .collect::<Vec<Cell>>(),
         )
         .height(header_height)
@@ -841,8 +853,8 @@ fn draw_table<B: Backend>(
     f.render_widget(table, layout_size);
 }
 
-fn draw_selection<B: Backend>(
-    f: &mut Frame<B>,
+fn draw_selection(
+    f: &mut Frame,
     _screen_size: TuiRect,
     layout_size: TuiRect,
     app: &app::App,
@@ -878,7 +890,10 @@ fn draw_selection<B: Backend>(
                 .unwrap_or_else(|| n.absolute_path.clone());
             string_to_text(out)
         })
-        .map(ListItem::new)
+        .map(|i| {
+            ListItem::new(i)
+                .style(app.config.general.selection.item.style.to_owned().into())
+        })
         .collect();
 
     // Selected items
@@ -894,8 +909,8 @@ fn draw_selection<B: Backend>(
     f.render_widget(selection_list, layout_size);
 }
 
-fn draw_help_menu<B: Backend>(
-    f: &mut Frame<B>,
+fn draw_help_menu(
+    f: &mut Frame,
     _screen_size: TuiRect,
     layout_size: TuiRect,
     app: &app::App,
@@ -924,25 +939,24 @@ fn draw_help_menu<B: Backend>(
         })
         .collect::<Vec<Row>>();
 
-    let help_menu = Table::new(help_menu_rows)
-        .block(block(
-            config,
-            format!(" Help [{}{}] ", &app.mode.name, read_only_indicator(app)),
-        ))
-        .widths(if app.config.general.hide_remaps_in_help_menu {
-            &[TuiConstraint::Percentage(20), TuiConstraint::Percentage(80)]
-        } else {
-            &[
-                TuiConstraint::Percentage(20),
-                TuiConstraint::Percentage(20),
-                TuiConstraint::Percentage(60),
-            ]
-        });
+    let widths = if app.config.general.hide_remaps_in_help_menu {
+        vec![TuiConstraint::Percentage(20), TuiConstraint::Percentage(80)]
+    } else {
+        vec![
+            TuiConstraint::Percentage(20),
+            TuiConstraint::Percentage(20),
+            TuiConstraint::Percentage(60),
+        ]
+    };
+    let help_menu = Table::new(help_menu_rows, widths).block(block(
+        config,
+        format!(" Help [{}{}] ", &app.mode.name, read_only_indicator(app)),
+    ));
     f.render_widget(help_menu, layout_size);
 }
 
-fn draw_input_buffer<B: Backend>(
-    f: &mut Frame<B>,
+fn draw_input_buffer(
+    f: &mut Frame,
     _screen_size: TuiRect,
     layout_size: TuiRect,
     app: &app::App,
@@ -973,7 +987,7 @@ fn draw_input_buffer<B: Backend>(
         let width = layout_size.width.max(offset_width) - offset_width;
         let scroll = input.visual_scroll(width.into()) as u16;
 
-        let input_buf = Paragraph::new(Spans::from(vec![
+        let input_buf = Paragraph::new(Line::from(vec![
             Span::styled(
                 app.input.prompt.to_owned(),
                 app.config.general.prompt.style.to_owned().into(),
@@ -983,7 +997,12 @@ fn draw_input_buffer<B: Backend>(
         .scroll((0, scroll))
         .block(block(
             config,
-            format!(" Input [{}{}] ", app.mode.name, read_only_indicator(app)),
+            format!(
+                " Input [{}{}]{} ",
+                app.mode.name,
+                read_only_indicator(app),
+                selection_indicator(app),
+            ),
         ));
 
         f.render_widget(input_buf, layout_size);
@@ -998,8 +1017,8 @@ fn draw_input_buffer<B: Backend>(
     };
 }
 
-fn draw_sort_n_filter<B: Backend>(
-    f: &mut Frame<B>,
+fn draw_sort_n_filter(
+    f: &mut Frame,
     _screen_size: TuiRect,
     layout_size: TuiRect,
     app: &app::App,
@@ -1114,14 +1133,14 @@ fn draw_sort_n_filter<B: Backend>(
         format!("({item_count}) ")
     };
 
-    let p = Paragraph::new(Spans::from(spans))
+    let p = Paragraph::new(Line::from(spans))
         .block(block(config, format!(" Sort & filter {item_count}")));
 
     f.render_widget(p, layout_size);
 }
 
-fn draw_logs<B: Backend>(
-    f: &mut Frame<B>,
+fn draw_logs(
+    f: &mut Frame,
     _screen_size: TuiRect,
     layout_size: TuiRect,
     app: &app::App,
@@ -1179,24 +1198,25 @@ fn draw_logs<B: Backend>(
     let logs_count = if logs_count == 0 {
         String::new()
     } else {
-        format!("({logs_count}) ")
+        format!(" ({logs_count})")
     };
 
     let logs_list = List::new(logs).block(block(
         config,
         format!(
-            " Logs {}[{}{}] ",
+            " Logs{} [{}{}]{} ",
             logs_count,
             app.mode.name,
             read_only_indicator(app),
+            selection_indicator(app)
         ),
     ));
 
     f.render_widget(logs_list, layout_size);
 }
 
-pub fn draw_nothing<B: Backend>(
-    f: &mut Frame<B>,
+pub fn draw_nothing(
+    f: &mut Frame,
     _screen_size: TuiRect,
     layout_size: TuiRect,
     app: &app::App,
@@ -1208,8 +1228,8 @@ pub fn draw_nothing<B: Backend>(
     f.render_widget(nothing, layout_size);
 }
 
-pub fn draw_dynamic<B: Backend>(
-    f: &mut Frame<B>,
+pub fn draw_dynamic(
+    f: &mut Frame,
     screen_size: TuiRect,
     layout_size: TuiRect,
     app: &app::App,
@@ -1232,8 +1252,8 @@ pub fn draw_dynamic<B: Backend>(
     draw_static(f, screen_size, layout_size, app, panel, lua);
 }
 
-pub fn draw_static<B: Backend>(
-    f: &mut Frame<B>,
+pub fn draw_static(
+    f: &mut Frame,
     screen_size: TuiRect,
     layout_size: TuiRect,
     app: &app::App,
@@ -1286,8 +1306,7 @@ pub fn draw_static<B: Backend>(
                 .map(|w| w.to_tui(screen_size, layout_size))
                 .collect::<Vec<TuiConstraint>>();
 
-            let content = Table::new(rows)
-                .widths(&widths)
+            let content = Table::new(rows, widths)
                 .column_spacing(col_spacing.unwrap_or(1))
                 .block(block(config, "".into()));
 
@@ -1326,9 +1345,9 @@ pub struct ContentRendererArg {
     pub layout_size: Rect,
 }
 
-pub fn draw_layout<B: Backend>(
+pub fn draw_layout(
     layout: Layout,
-    f: &mut Frame<B>,
+    f: &mut Frame,
     screen_size: TuiRect,
     layout_size: TuiRect,
     app: &app::App,
@@ -1422,7 +1441,7 @@ pub fn draw_layout<B: Backend>(
     }
 }
 
-pub fn draw<B: Backend>(f: &mut Frame<B>, app: &app::App, lua: &Lua) {
+pub fn draw(f: &mut Frame, app: &app::App, lua: &Lua) {
     let screen_size = f.size();
     let layout = app.mode.layout.as_ref().unwrap_or(&app.layout).to_owned();
 
